﻿
#region Header

/* /////////////////////////////////////////////////////////////////////////////////////////////////
XpVRControllers for Virt-A-Mate by Fabio Den.

Written in Visual Studio
	
Version 1.0 2022-03-09
	Initial release.

License: CC BY-SA
	
///////////////////////////////////////////////////////////////////////////////////////////////// */

using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using UnityEngine.UI;
using UnityEngine.Events;
using UnityEngine.Animations;
using System.Linq;
using MVR.FileManagementSecure;
using SimpleJSON;

#endregion

public class VRControllersFree : MVRScript
{

    #region Variables

    private JSONStorableBool enabledChk;
    private float tolerance = 0.1f;
    private float tolerancer = 0.35f;
    private bool changing = false;
    private bool changing2 = false;
    private bool wasLB = false;
    //private int sep = 5;

    private string leftAtom = "";
    private string rightAtom = "";
    private FreeControllerV3 leftObject = null;	 		// selected left controller
    private FreeControllerV3 rightObject = null;	 	// selected right controller
    private int statusL = 1;
    private int statusR = 1;
    private string targetAtom = "";
    private string targetCtrl = "";

    private UIDynamicButton myLabelL;
    private UIDynamicButton myLabelR;

    private JSONStorableBool localCoordL;
    private JSONStorableBool localCoordR;

    private JSONStorableFloat moveSpeedL;
    private JSONStorableFloat moveSpeedR;
    private JSONStorableFloat rotSpeedL;
    private JSONStorableFloat rotSpeedR;

    private JSONStorableBool autoPlayEdit;

    private Vector3 initPosOL;
    private Vector3 initPosOR;
    private Quaternion initRotOL;
    private Quaternion initRotOR;

    private Vector3 initPosL;
    private Vector3 initPosR;
    private Quaternion initRotL;
    private Quaternion initRotR;

    public int counter = 0;
    protected List<MyCheckbox> checkboxes;

    private JSONStorableBool[] Joints = new JSONStorableBool[8];

    #endregion

    #region Events

    public override void Init()
    {
        checkboxes = new List<MyCheckbox>();
    }

    public void OnEnable()
    {
        SuperController.singleton.onSceneLoadedHandlers += OnSceneLoaded;
    }

    public void OnDisable()
    {
        SuperController.singleton.onSceneLoadedHandlers -= OnSceneLoaded;
    }

    public void Start()
    {
        SetupGUI();
    }

    protected void OnSceneLoaded()
    {
        leftAtom = "";
        rightAtom = "";

        targetAtom = "";
        targetCtrl = "";

        leftObject = null;
        rightObject = null;

        statusL = 1;
        statusR = 1;

        OnSync();

        UpdateStatus();
    }

    public void OnDestroy()
    {
        if (enabledChk.val)
        {
            SuperController.singleton.ToggleDisableAllNavigation();
        }
    }

    #endregion

    #region GUI

    private void SetupGUI()
    {
        UIDynamicButton btn;
        int fontsize = 30;
        ButtonDeck deck;

        enabledChk = GUICreateCheckbox(this, "Enabled", "Enabled", false, false, SyncEnabled);
        autoPlayEdit = GUICreateCheckbox(this, "disableRoot", "Auto-toggle <b>Edit / Play</b> modes", false, true, SyncRoot);

        moveSpeedL = GUICreateSlider(this, "moveSpeedL", "Left Movement Speed", 35.0F, false, 0F, 100.0F);
        moveSpeedR = GUICreateSlider(this, "moveSpeedR", "Right Movement Speed", 35.0F, true, 0F, 100.0F);

        rotSpeedL = GUICreateSlider(this, "rotSpeedL", "Left Rotation Speed", 25.0F, false, 0F, 100.0F);
        rotSpeedR = GUICreateSlider(this, "rotSpeedR", "Right Rotation Speed", 25.0F, true, 0F, 100.0F);

        localCoordL = GUICreateCheckbox(this, "localCoordL", "Local Coordinates", false, false, SyncCoordL);
        localCoordR = GUICreateCheckbox(this, "localCoordR", "Local Coordinates", false, true, SyncCoordR);

        myLabelL = GUICreateButton(this, "", new Color(0.78F, 0.78F, 0.899F), false, false, OnStatusL);
        myLabelL.height = 90;

        myLabelR = GUICreateButton(this, "", new Color(0.78F, 0.78F, 0.899F), false, true, OnStatusR);
        myLabelR.height = 90;

        deck = new ButtonDeck(this, "Map to\nLeft Controller", "Clear", false, OnMapToLeftTarget, OnClearLeft);
        deck = new ButtonDeck(this, "Map to\nRight Controller", "Clear", true, OnMapToRightTarget, OnClearRight);
        
        UpdateStatus();

        //CreateSpacer(false).height = sep;
        //CreateSpacer(true).height = sep;

        btn = GUICreateButton(this, "Synchronize\nTargets", new Color(0.78F, 0.78F, 0.899F), true, false, OnSync);
        btn.height = 115.0F;
        SetFontSize(btn, fontsize);

        for (int i = 1; i <= Joints.Length - 1; i++)
        {
            int x = i; // each index needs its own int
            Joints[i] = GUICreateCheckbox(this, "Joint" + i.ToString(), GetJointName(i), false, true, (JSONStorableBool flag) => SyncJoint(flag, x));
        }
    }

    private void OnStatusL()
    {
        statusL += 1;
        if (statusL == 4) { statusL = 1; }
        UpdateStatus();
    }

    private void OnStatusR()
    {
        statusR += 1;
        if (statusR == 4) { statusR = 1; }
        UpdateStatus();
    }

    private void SyncJoint(JSONStorableBool flag, int j)
    {
        if (flag.val)
        {
            for (int i = 1; i <= Joints.Length - 1; i++)
            {
                if (i != j) { Joints[i].val = false; }
            }
        }

        Joints[j].val = flag.val;

        if (flag.val)
        {
            targetCtrl = GetJointName(j);
            targetCtrl = targetCtrl.Replace("<b>", "");
            targetCtrl = targetCtrl.Replace("</b>", "");
        }
        else
        {
            targetCtrl = "";
        }
    }

    private string GetJointName(int i)
    {
        string name = "";

        switch (i)
        {
            case 1:
                name = "<b>head</b>Control";
                break;
            case 2:
                name = "<b>chest</b>Control";
                break;
            case 3:
                name = "<b>hip</b>Control";
                break;
            case 4:
                name = "<b>lHand</b>Control";
                break;
            case 5:
                name = "<b>rHand</b>Control";
                break;
            case 6:
                name = "<b>lFoot</b>Control";
                break;
            case 7:
                name = "<b>rFoot</b>Control";
                break;
        }

        return name;
    }

    public void OnSync()
    {
        foreach (MyCheckbox slot in checkboxes)
        {
            slot.Remove();
        }

        foreach (Atom a in SuperController.singleton.GetAtoms())
        {
            if (a.category == "People")
                checkboxes.Add(new MyCheckbox(a.name, this));
        }

        for (int i = 1; i <= Joints.Length - 1; i++)
        {
            Joints[i].val = false;
        }
    }

    public static JSONStorableStringChooser GUICreatePopup(MVRScript dll, string name, string label, bool rightside, UIPopup.OnOpenPopup popup, JSONStorableStringChooser.SetStringCallback callback)
    {
        JSONStorableStringChooser gui = new JSONStorableStringChooser(name, null, "", "", callback);
        dll.RegisterStringChooser(gui);
        UIDynamicPopup dp = dll.CreateFilterablePopup(gui, rightside);
        dp.popupPanelHeight = 600.0F;
        dp.label = label;
        dp.popup.onOpenPopupHandlers = popup;

        return gui;
    }

    private void SetFontSize(UIDynamicButton btn, int fontsize)
    {
        Text text_1 = btn.button.GetComponentInChildren<UnityEngine.UI.Text>();
        if (text_1) { text_1.fontSize = fontsize; }
    }

    private void OnPluginButton()
    {
        //pluginButton.label = pluginName + pluginversion;
    }

    private void OnReturnButton()
    {
        ShowPlugin("VAMPortal");
    }

    private void OnReserved()
    {

    }

    public static UIDynamicButton GUICreateButton(MVRScript dll, string label, Color _color, bool useColor, bool rightside, UnityAction callback)
    {
        UIDynamicButton gui = dll.CreateButton(label, rightside);
        if (useColor) { gui.buttonColor = _color; }
        gui.button.onClick.AddListener(callback);

        return gui;
    }

    public static JSONStorableFloat GUICreateSlider(MVRScript dll, string name, string label, float startvalue, bool rightside, float minvalue, float maxvalue)
    {
        JSONStorableFloat gui = new JSONStorableFloat(name, startvalue, minvalue, maxvalue);
        var Slider = dll.CreateSlider(gui, rightside);
        Slider.label = label;
        dll.RegisterFloat(gui);

        return gui;
    }

    public static JSONStorableBool GUICreateCheckbox(MVRScript dll, string name, string label, bool startvalue, bool rightside, JSONStorableBool.SetJSONBoolCallback callback)
    {
        JSONStorableBool gui = new JSONStorableBool(name, startvalue, callback);
        var Toggle = dll.CreateToggle(gui, rightside);
        Toggle.label = label;

        return gui;
    }

    #endregion

    #region Deck

    public class ButtonDeck
    {
        public UIDynamicButton uiButton1;
        public UIDynamicButton uiButton2;
        public VRControllersFree parent;

        public ButtonDeck(VRControllersFree parent, string name1, string name2, bool rightside, UnityAction callback1, UnityAction callback2)
        {
            this.parent = parent;
            var toolbar = new GameObject();

            if (rightside)
                toolbar.transform.SetParent(parent.rightUIContent.transform, false);
            else
                toolbar.transform.SetParent(parent.leftUIContent.transform, false);

            var layout = toolbar.AddComponent<HorizontalLayoutGroup>();
            layout.spacing = 8;

            uiButton1 = CreateButton(layout.transform, name1, false, callback1);
            uiButton2 = CreateButton(layout.transform, name2, false, callback2);
        }

        public void SetText(int number, string label)
        {
            UIDynamicButton btn = uiButton1;
            if (number == 2) { btn = uiButton2; }
            btn.label = label;
        }

        public void SetColor(int number, int colorcode)
        {
            UIDynamicButton btn = uiButton1;
            if (number == 2) { btn = uiButton2; }

            switch (colorcode)
            {
                case 1: // normal
                    btn.buttonColor = new Color(0.78F, 0.78F, 0.899F);
                    break;

                case 2: // green
                    btn.buttonColor = new Color(0.780f, 0.899f, 0.780f);
                    break;

                case 3: // yellow
                    btn.buttonColor = new Color(0.255f / 0.255f, 0.244f / 0.255f, 0.174f / 0.255f);
                    break;

                case 4: // gray
                    btn.buttonColor = new Color32(205, 205, 205, 255);
                    break;
            }
        }

        public UIDynamicButton CreateButton(Transform t, string name, bool rightside, UnityAction callback)
        {
            Transform button = GameObject.Instantiate<Transform>(parent.manager.configurableButtonPrefab);
            button.SetParent(t, false);
            UIDynamicButton uiButton = button.GetComponent<UIDynamicButton>();
            uiButton.label = name;
            uiButton.buttonText.fontSize = 30;
            uiButton.height = 115.0F;
            uiButton.buttonColor = new Color(0.78F, 0.78F, 0.899F);
            uiButton.button.onClick.AddListener(callback);
            return uiButton;
        }
    }

    #endregion

    #region	Plugin Visibility

    private bool ShowPlugin(string name)
    {
        bool found = false;

        MVRScriptControllerUI[] ts = this.manager.pluginListPanel.GetComponentsInChildren<MVRScriptControllerUI>();

        foreach (MVRScriptControllerUI t in ts)
        {
            if (t.label.text.Contains(name))
            {
                HidePlugin();
                t.openUIButton.onClick.Invoke();
                found = true;
                break;
            }
        }

        return found;
    }

    private void HidePlugin()
    {
        this.UITransform.gameObject.SetActive(value: false);
    }

    #endregion

    #region	Status

    private void UpdateStatus()
    {
        string leftCont = "";
        string rightCont = "";

        if (leftObject == null)
        {
            leftAtom = "Not assigned";
        }
        else
        {
            leftCont = leftObject.name;
        }

        if (rightObject == null)
        {
            rightAtom = "Not assigned";
        }
        else
        {
            rightCont = rightObject.name;
        }

        char symbol = '\u25FE';

        string rotL = "\n<b>Move</b> " + symbol.ToString() + " Rotate " + symbol.ToString() + " Possess";
        if (statusL == 2) { rotL = "\nMove " + symbol.ToString() + " <b>Rotate</b> " + symbol.ToString() + " Possess"; }
        if (statusL == 3) { rotL = "\nMove " + symbol.ToString() + " Rotate " + symbol.ToString() + " <b>Possess</b>"; }
        myLabelL.label = "<b>" + leftAtom + "</b> " + symbol.ToString() + " " + leftCont + rotL;

        string rotR = "\n<b>Move</b> " + symbol.ToString() + " Rotate " + symbol.ToString() + " Possess";
        if (statusR == 2) { rotR = "\nMove " + symbol.ToString() + " <b>Rotate</b> " + symbol.ToString() + " Possess"; }
        if (statusR == 3) { rotR = "\nMove " + symbol.ToString() + " Rotate " + symbol.ToString() + " <b>Possess</b>"; }
        myLabelR.label = "<b>" + rightAtom + "</b> " + symbol.ToString() + " " + rightCont + rotR;
    }

    #endregion

    #region	Mapping

    private void OnMapToLeftTarget()
    {
        Atom cur = SuperController.singleton.GetAtomByUid(targetAtom);
        FreeControllerV3 controller = null;

        if (cur)
        {
            if (targetCtrl.Equals("")) { targetCtrl = "control"; }
            controller = cur.GetStorableByID(targetCtrl) as FreeControllerV3; ;
        }
        else
        {
            cur = SuperController.singleton.GetSelectedAtom();
            if (cur == null) { return; }
            controller = SuperController.singleton.GetSelectedController();
        }

        if (controller == null) { return; }

        leftAtom = cur.name;
        leftObject = controller;
        UpdateStatus();
    }

    private void OnMapToRightTarget()
    {
        Atom cur = SuperController.singleton.GetAtomByUid(targetAtom);
        FreeControllerV3 controller = null;

        if (cur)
        {
            if (targetCtrl.Equals("")) { targetCtrl = "control"; }
            controller = cur.GetStorableByID(targetCtrl) as FreeControllerV3; ;
        }
        else
        {
            cur = SuperController.singleton.GetSelectedAtom();
            if (cur == null) { return; }
            controller = SuperController.singleton.GetSelectedController();
        }

        if (controller == null) { return; }

        rightAtom = cur.name;
        rightObject = controller;
        UpdateStatus();
    }

    private void OnClearLeft()
    {
        leftAtom = "";
        leftObject = null;
        statusL = 1;

        UpdateStatus();
    }

    private void OnClearRight()
    {
        rightAtom = "";
        rightObject = null;
        statusR = 1;

        UpdateStatus();
    }

    #endregion

    #region	States

    private float GetSpeed(bool rightSide)
    {
        if (rightSide) { return moveSpeedR.val; }
        return moveSpeedL.val;
    }

    private bool usingLocal(bool rightSide)
    {
        if (rightSide) { return localCoordR.val; }
        return localCoordL.val;
    }

    private FreeControllerV3 GetObject(bool rightSide)
    {
        FreeControllerV3 controller = SuperController.singleton.GetSelectedController();

        if ((leftObject != null) || (rightObject != null))
        {
            // controller = null;   // confusing either way
        }

        if (rightSide)
        {
            if (rightObject != null) { controller = rightObject; }
        }
        else
        {
            if (leftObject != null) { controller = leftObject; }
        }

        return controller;
    }

    private float GetRotSpeed(bool rightSide)
    {
        if (rightSide) { return rotSpeedR.val; }
        return rotSpeedL.val;
    }

    #endregion

    #region	Movement

    public void ControllerMove(int dir, float up, bool rightSide)
    {
        FreeControllerV3 controller = GetObject(rightSide);
        if (controller == null) { return; }

        var direction = Vector3.right;

        switch (dir)
        {
            case 1:
                direction = Vector3.right;
                break;
            case 2:
                direction = Vector3.up;
                break;
            case 3:
                direction = Vector3.forward;
                break;
        }

        float speed = GetSpeed(rightSide) / 100F;
        bool local = usingLocal(rightSide);

        if (local)
            controller.transform.Translate(direction * Time.deltaTime * up * speed, Space.Self);
        else
            controller.transform.Translate(direction * Time.deltaTime * up * speed, Camera.main.transform);
    }

    #endregion

    #region	Rotation

    public void ControllerRotate(int dir, float up, bool rightSide)
    {
        float speed = GetRotSpeed(rightSide) / 100F;
        bool local = usingLocal(rightSide);

        var rotate = 360 * Time.deltaTime * up * speed;

        FreeControllerV3 target = GetObject(rightSide);
        if (target == null) { return; }

        if (local)
        {
            if (dir == 1) { target.transform.Rotate(rotate, 0, 0, Space.Self); } // yes
            else if (dir == 2) { target.transform.Rotate(0, rotate, 0, Space.Self); }// no
            else if (dir == 3) { target.transform.Rotate(0, 0, rotate, Space.Self); } // roll	
        }
        else
        {
            if (dir == 1) { target.transform.RotateAround(target.transform.position, Camera.main.transform.right, -rotate); } // yes
            else if (dir == 2) { target.transform.RotateAround(target.transform.position, Camera.main.transform.up, rotate); } // no
            else if (dir == 3) { target.transform.RotateAround(target.transform.position, Camera.main.transform.forward, rotate); } // roll
        }
    }

    #endregion

    #region	Sticks

    private void ProcessLeftStick(bool a, Vector4 axes)
    {
        if (a == false)
        {
            // LEFT STICK HORIZONTAL
            if (statusL == 1)
            {
                if (Math.Abs(axes.x) > tolerance)
                {
                    ControllerMove(1, axes.x, false);
                }
            }
            else
            {
                if (Math.Abs(axes.x) > tolerancer)
                {
                    ControllerRotate(2, -axes.x, false);
                }
            }

            // LEFT STICK VERTICAL
            if (statusL == 1)
            {
                if (Math.Abs(axes.y) > tolerance)
                {
                    ControllerMove(2, axes.y, false);
                }
            }
            else
            {
                if (Math.Abs(axes.y) > tolerancer)
                {
                    ControllerRotate(1, -axes.y, false);
                }
            }
        }
        else
        {
            if (statusL == 1)
            {
                // LEFT STICK VERTICAL
                if (Math.Abs(axes.y) > tolerance)
                {
                    ControllerMove(3, axes.y, false);
                }
            }
            else
            {
                // LEFT STICK HORIZONTAL
                if (Math.Abs(axes.x) > tolerancer)
                {
                    ControllerRotate(3, -axes.x, false);
                }
            }
        }
    }

    private void ProcessRightStick(bool b, Vector4 axes)
    {
        if (b == false)
        {
            // RIGHT STICK HORIZONTAL
            if (statusR == 1)
            {
                if (Math.Abs(axes.z) > tolerance)
                {
                    ControllerMove(1, axes.z, true);
                }
            }
            else
            {
                if (Math.Abs(axes.z) > tolerancer)
                {
                    ControllerRotate(2, -axes.z, true);
                }
            }


            // RIGHT STICK VERTICAL
            if (statusR == 1)
            {
                if (Math.Abs(axes.w) > tolerance)
                {
                    ControllerMove(2, axes.w, true);
                }
            }
            else
            {
                if (Math.Abs(axes.w) > tolerancer)
                {
                    ControllerRotate(1, -axes.w, true);
                }
            }
        }
        else
        {
            if (statusR == 1)
            {
                // RIGHT STICK VERTICAL
                if (Math.Abs(axes.w) > tolerance)
                {
                    ControllerMove(3, axes.w, true);
                }
            }
            else
            {
                // RIGHT STICK HORIZONTAL
                if (Math.Abs(axes.z) > tolerancer)
                {
                    ControllerRotate(3, -axes.z, true);
                }
            }
        }
    }

    #endregion

    #region	Flags

    private void SyncRoot(JSONStorableBool flag)
    {
        autoPlayEdit.val = flag.val;
    }

    private void SyncCoordL(JSONStorableBool flag)
    {
        localCoordL.val = flag.val;
    }

    private void SyncCoordR(JSONStorableBool flag)
    {
        localCoordR.val = flag.val;
    }

    private void SyncEnabled(JSONStorableBool flag)
    {
        enabledChk.val = flag.val;

        SuperController.singleton.ToggleDisableAllNavigation();

        initPosL = SuperController.singleton.leftHand.position;
        initRotL = SuperController.singleton.leftHand.rotation;

        if (leftObject)
        {
            initPosOL = leftObject.transform.position;
            initRotOL = leftObject.transform.rotation;
        }

        initPosR = SuperController.singleton.rightHand.position;
        initRotR = SuperController.singleton.rightHand.rotation;

        if (rightObject)
        {
            initPosOR = rightObject.transform.position;
            initRotOR = rightObject.transform.rotation;
        }
    }

    #endregion

    #region CheckBoxes

    public class MyCheckbox
    {
        public string name;
        public VRControllersFree parent;
        public JSONStorableBool _check;

        public MyCheckbox(string name, VRControllersFree parent)
        {
            this.name = name;
            this.parent = parent;
            CreateCheckBox();
        }

        public void CreateCheckBox()
        {
            parent.counter++;
            _check = VRControllersFree.GUICreateCheckbox(parent, "check" + parent.counter.ToString(), "<b>" + name + "</b>", false, false, SyncCheckBox);
        }

        private void SyncCheckBox(JSONStorableBool flag)
        {
            if (flag.val)
            {
                foreach (MyCheckbox slot in parent.checkboxes)
                {
                    if (slot != this)
                        slot.SetValue(false);
                }
            }

            _check.val = flag.val;

            if (_check.val)
            {
                parent.targetAtom = name;
                _check.val = true;
            }
        }

        public void SetValue(bool _val)
        {
            _check.val = _val;
        }

        public bool GetValue()
        {
            return _check.val;
        }

        public void Remove()
        {
            parent.RemoveToggle(_check);
        }
    }

    #endregion

    #region	Possession

    private void ProcessLeftPossession(bool LT)
    {
        if (statusL == 3)
        {
            if (LT)
            {
                leftObject.transform.position = initPosOL + SuperController.singleton.leftHand.position - initPosL;
                leftObject.transform.rotation = SuperController.singleton.leftHand.rotation * Quaternion.Inverse(initRotL) * initRotOL;
            }
            else
            {
                initPosL = SuperController.singleton.leftHand.position;
                initRotL = SuperController.singleton.leftHand.rotation;

                initPosOL = leftObject.transform.position;
                initRotOL = leftObject.transform.rotation;
            }
        }
    }

    private void ProcessRightPossession(bool RT)
    {
        if (statusR == 3)
        {
            if (RT)
            {
                rightObject.transform.position = initPosOR + SuperController.singleton.rightHand.position - initPosR;
                rightObject.transform.rotation = SuperController.singleton.rightHand.rotation * Quaternion.Inverse(initRotR) * initRotOR;
            }
            else
            {
                initPosR = SuperController.singleton.rightHand.position;
                initRotR = SuperController.singleton.rightHand.rotation;

                initPosOR = rightObject.transform.position;
                initRotOR = rightObject.transform.rotation;
            }
        }
    }

    #endregion

    #region Update

    void FixedUpdate()
    {
        DoStep();
    }

    private bool menu_opened = false;

    private void DoStep()
    {
        bool LB = SuperController.singleton.GetLeftHoldGrab();
        bool RB = SuperController.singleton.GetRightHoldGrab();
        bool LT = SuperController.singleton.GetLeftGrabVal() > tolerance;
        bool RT = SuperController.singleton.GetRightGrabVal() > tolerance;
        bool Y = SuperController.singleton.GetMenuMoveLeft();  // Y: left show/hide menu	- return TRUE while the button is being pressed
        bool B = SuperController.singleton.GetMenuMoveRight(); // B: right show/hide menu	- return TRUE while the button is being pressed

        if (Y || B)
        {
            if (menu_opened == false)
            {
                menu_opened = true;

                // set edit mode
                if (autoPlayEdit.val)
                {
                    if (SuperController.singleton.mainHUD.gameObject.activeSelf)
                        SuperController.singleton.gameMode = SuperController.GameMode.Edit;
                    else
                        SuperController.singleton.gameMode = SuperController.GameMode.Play;
                }
            }
        }
        else
        {
            menu_opened = false;
        }

        // Enable - disable the plugin
        if ((LB && LT) || (RB && RT))
        {
            if (changing == false)
            {
                changing = true;
                enabledChk.val = !enabledChk.val;
                changing2 = false;
            }

            return;
        }

        // Preparing a future toggle move - rotate
        if (LB || RB)
        {
            if (changing2 == false)
            {
                changing2 = true;

                if (LB)
                {
                    wasLB = true;
                }

                if (RB)
                {
                    wasLB = false;
                }
            }

            return;
        }

        // toggle move - rotate
        if ((changing2 == true) && (changing == false))
        {
            if (wasLB)
            {
                statusL += 1;
                if (statusL == 4) { statusL = 1; }
                UpdateStatus();
            }
            else
            {
                statusR += 1;
                if (statusR == 4) { statusR = 1; }
                UpdateStatus();
            }
        }

        // if nothing else was pressed proceed with the sticks
        if ((changing2 == false) && (changing == false))
        {
            if (enabledChk.val)
            {
                Vector4 axes = SuperController.singleton.GetFreeNavigateVector(SuperController.singleton.freeMoveAction);

                if (statusL <= 2)
                {
                    ProcessLeftStick(LT, axes);
                }
                else
                {
                    ProcessLeftPossession(LT);
                }

                if (statusR <= 2)
                {
                    ProcessRightStick(RT, axes);
                }
                else
                {
                    ProcessRightPossession(RT);
                }
            }
        }

        changing2 = false;
        changing = false;
    }

    #endregion

} // End class

